# @(#) $Id: VCalendarRule.pm,v 1.2 2002/05/30 11:56:19 nick Exp $
#	Copyright (C) 2002, Nick Andrew <nick@nick-andrew.net>
#	Distributed under the terms of the GNU Public License
#	See http://www.gnu.org/licenses/gpl.html
#
#  A VCalendarRule is a set of parameters which define when a repeating
#  event occurs.

package VCalendarRule;

use Time::Local;

sub new {
	my $class = shift;
	my $self = { };

	bless $self, $class;

	$self->{rule_string} = shift;

	$self->parse();

	return $self;
}

sub yyyymmdd_to_iso {
	$_ = shift;

	if (/^(\d\d\d\d)(\d\d)(\d\d)$/) {
		my $iso_date = "$1-$2-$3";
		my $iso_time = "$4:$5:$6";

		return $iso_date;
	}

	return undef;
}

sub dddt_to_iso {
	my $dddt = shift;

	if (/^(\d\d\d\d)(\d\d)(\d\d)T(\d\d)(\d\d)(\d\d)$/) {
		my $iso_date = "$1-$2-$3";
		my $iso_time = "$4:$5:$6";

		return ($iso_date, $iso_time);
	}

	return (undef,undef);
}


sub parse {
	my $self = shift;

	# I only know how to parse these rules:
	# RRULE:W1 SA #0 
	# RRULE:W1 MO TU WE TH FR 20020706T000000 
	# RRULE:W1 MO TU WE TH 20020706T000000 

	my $rrule = $self->{rule_string} || die "No rule string defined";
	my $need_dow = 0;

	if ($rrule =~ /^W1 /) {
		my @words = split(/\s+/, $rrule);

		foreach (@words) {
			if (/^(SU|MO|TU|WE|TH|FR|SA)$/) {
				# day-of-week required
				$need_dow = 1;
				$self->{dow}->{$1} = 1;
				next;
			}

			if (/^W1$/) {
				# Ignore, we know what this means
				next;
			}

			if (/^#0$/) {
				# I think it means go forever
				next;
			}

			if (/^\d\d\d\d\d\d\d\dT\d\d\d\d\d\d$/) {
				my ($iso_date,$iso_time) = dddt_to_iso($_);
				$self->{end_date} = $iso_date;
				$self->{end_time} = $iso_time;
				next;
			}

			die "Warning: unknown rule fragment: $_ in $rrule\n";
		}
	} elsif ($rrule =~ /^FREQ=WEEKLY/) {
		# FREQ=WEEKLY;INTERVAL=1;BYDAY=MO
		# FREQ=WEEKLY;UNTIL=20020706;INTERVAL=1;BYDAY=MO,TU,WE,TH
		my @a = split(/;/, $rrule);
		my %h;
#		print STDERR "a is @a\n";

		foreach (@a) {
			if (/^([^=]+)=(.+)/) {
				$h{$1} = $2;
			}
		}

		foreach my $key (keys %h) {
			my $value = $h{$key};

			if ($key eq 'FREQ') {
				die "Unknown frequency: $value in $rrule\n" if ($value ne 'WEEKLY');
				next;
			}

			if ($key eq 'BYDAY') {
				my @days = split(/,/, $value);
				foreach (@days) {
					$self->{dow}->{$_} = 1;
				}
				next;
			}

			if ($key eq 'INTERVAL') {
				die "Unable to handle interval: $value in $rrule\n" if ($value != 1);
				next;
			}

			if ($key eq 'UNTIL') {
				$self->{end_date} = yyyymmdd_to_iso($value);
				# Not sure if it includes the date
				$self->{end_time} = '23:59:59';
				next;
			}

			die "Unknown key: $key in $rrule\n";
		}

	} else {
		die "Unparseable rule: $rrule\n";
	}

}

sub applies_to_date {
	my $self = shift;
	my $target_date = shift;

#	print STDERR "Checking rule against target date $target_date\n";

	if (exists $self->{end_date}) {
		# Make sure target isn't >= the end date

#		print STDERR "\tend_date exists and is $self->{end_date}\n";

		# FIXME ... what happens when end_time ne '00:00:00' ??
		if ($target_date ge $self->{end_date}) {
#			print STDERR "\tReturning 0 because target_date >= end_date\n";
			return 0;
		}
	}

	if (exists $self->{dow}) {
		# Figure out the day of week of the target date

		# FIXME performance ... better to have a Date class
		# FIXME ... and figure out the DOW only once

		my($sec,$min,$hour,$mday,$mon,$year);

		($sec,$min,$hour) = (0,0,0);
		($year,$mon,$mday) = ($target_date =~ /(\d\d\d\d)-(\d\d)-(\d\d)/);
		$year -= 1900; $mon--;
		my $ts = timelocal($sec,$min,$hour,$mday,$mon,$year);
		# Now reverse the action
		my($n_sec,$n_min,$n_hour,$n_mday,$n_mon, $n_year, $n_wday) = localtime($ts);

		warn "seconds mismatch: expected $sec, got $n_sec (ts=$ts)" if ($n_sec != $sec);
		warn "minutes mismatch: expected $min, got $n_min (ts=$ts)" if ($n_min != $min);
		warn "hours mismatch: expected $hour, got $n_hour (ts=$ts)" if ($n_hour != $hour);
		warn "mday mismatch: expected $mday, got $n_mday (ts=$ts)" if ($n_mday != $mday);
		warn "month mismatch: expected $mon, got $n_mon (ts=$ts)" if ($n_mon != $mon);
		warn "year mismatch: expected $year, got $n_year (ts=$ts)" if ($n_year != $year);

		my $dow = {0=>'SU', 1=>'MO', 2=>'TU', 3=>'WE', 4=>'TH', 5=>'FR', 6=>'SA'} -> {$n_wday};

#		print STDERR "\tdow of $target_date is $dow\n";

		if (!exists $self->{'dow'}->{$dow}) {
#			print STDERR "\treturning 0 because $dow is not in the rule\n";
			return 0;
		}
	}

	# All requirements have been satisfied, so it's okay

#	print STDERR "\tIt's okay!\n";

	return 1;
}

1;
